from markdown.extensions.toc import TocExtension, TocTreeprocessor
from markdown.util import etree
from six import text_type

class _OrderedTocTreeprocessor(TocTreeprocessor):

    def __init__(self, md, config):

        super(_OrderedTocTreeprocessor, self).__init__(md, config)

        # Ensure that the title is a real string, not a proxy
        self.title = text_type(self.title)

    def build_toc_div(self, toc_list):
        """
        Return a string div given a toc list.
        """

        div = etree.Element('div')
        div.attrib['class'] = "toc"

        # Add title to the div
        if self.title:
            header = etree.SubElement(div, "div")
            header.attrib['class'] = "toctitle"
            header.text = self.title

        def build_etree_ol(toc_list, parent):
            ol = etree.SubElement(parent, 'ol')
            for item in toc_list:
                # List item link, to be inserted into the toc div
                li = etree.SubElement(ol, 'li')
                link = etree.SubElement(li, 'a')
                link.text = item.get('name', '')
                link.attrib['href'] = '#' + item.get('id', '')
                if item['children']:
                    build_etree_ol(item['children'], li)

            return ol

        build_etree_ol(toc_list, div)
        prettify = self.markdown.treeprocessors.get('prettify')
        if prettify:
            prettify.run(div)

        return div

class OrderedTocExtension(TocExtension):
    """
    An improved TOC for markdown which uses an ordered list for the table of
    contents.
    """

    TreeProcessorClass = _OrderedTocTreeprocessor

def makeExtension(*args, **kwargs):

    return OrderedTocExtension(*args, **kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-01-06 01:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('thorson_wiki', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='article',
            name='login_required',
        ),
        migrations.AddField(
            model_name='article',
            name='public',
            field=models.BooleanField(default=True, help_text='Whether the article is completely public.', verbose_name='public'),
        ),
        migrations.AlterField(
            model_name='article',
            name='namespace',
            field=models.CharField(choices=[('srd5', '5th Edition SRD'), ('rivenrock', 'The Rivenrock Wiki')], help_text="The article's namespace.", max_length=255, verbose_name='namespace'),
        ),
    ]



from thorson_wiki.models import Article
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models.signals import pre_save, pre_delete, post_save
from django.dispatch import receiver
from django.utils.text import slugify

@receiver(pre_save, sender=Article)
def update_slug(sender, **kwargs):
    """
    Updates an Articles slug when the object is updated. This also results in
    updating the article permissions.
    """

    instance = kwargs['instance']
    try:
        old_instance = Article.objects.get(id=instance.id)
    except Article.DoesNotExist:
        old_instance = None

    instance.slug = slugify(instance.title)

    if old_instance is None:
        return

    if instance.slug != old_instance.slug:
        edit_permission = Permission.objects.get(
            codename='edit-article:%s' % old_instance.slug
        )

        edit_permission.codename='edit-article:%s' % instance.slug
        edit_permission.name="Can edit the article '%s'" % instance.title
        edit_permission.save()

        read_permission = Permission.objects.get(
            codename='read-article:%s' % old_instance.slug
        )

        read_permission.codename='read-article:%s' % instance.slug
        read_permission.name="Can read the article '%s'" % instance.title
        read_permission.save()

@receiver(post_save, sender=Article)
def create_article_permissions(sender, **kwargs):
    """
    Creates individual article permissions when an article is created.
    """

    instance = kwargs['instance']
    created = kwargs['created']

    slug = instance.slug
    title = instance.title

    if created:
        content_type = ContentType.objects.get_for_model(Article)
        permission = Permission.objects.create(
            codename='edit-article:%s' % slug,
            name="Can edit the article '%s'" % title,
            content_type=content_type
        )

        content_type = ContentType.objects.get_for_model(Article)
        permission = Permission.objects.create(
            codename='read-article:%s' % slug,
            name="Can read the article '%s'" % title,
            content_type=content_type
        )

@receiver(pre_delete, sender=Article)
def delete_article_permissions(sender, **kwargs):
    """
    Deletes an article's permissions when the article is deleted.
    """

    instance = kwargs['instance']

    slug = instance.slug

    Permission.objects.get(codename='edit-article:%s' % slug).delete()
    Permission.objects.get(codename='read-article:%s' % slug).delete()



"""
Generally, each wiki on a website will want its own set of URLs. These can be
created in a Django URL conf using the :func:`include_wiki` function, which
produces a set of URLs for the given wiki namespace. 
"""

from django.conf import settings
from django.conf.urls import include, url
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse_lazy
from django.utils.text import slugify
from thorson_wiki import views

import django.contrib.auth.views as auth_views

def include_wiki(namespace_):
    """
    :param namespace_: the name of the wiki to include
    :rtype namespace_: str

    Creates a set of URLs for the given namespace. This is used much like
    :func:`django.conf.urls.include`, except it is given a string, which should
    correspond to a namespace your settings.
    """

    if namespace_ not in getattr(settings, 'THORSONWIKI_NAMESPACES', {}):
        raise ImproperlyConfigured(("Included namespace '%s' not in" \
                " Django settings.") % namespace_)

    return include((generate_urls(namespace_), 'thorson_wiki'), namespace_)

def generate_urls(namespace_):
    """
    Returns a list of URLs for the given namespace.
    """

    namespaces = getattr(settings, 'THORSONWIKI_NAMESPACES', {})

    try:
        namespace = namespaces[namespace_]
    except KeyError:
        raise ImproperlyConfigured("Invalid namespace: '%s'" % namespace_)

    templates = namespace.get('templates', {})
    login_template = templates.get('login', 'thorson_wiki/login.html')
    password_change_template = templates.get('password_change',
            'thorson_wiki/password_change.html')
    password_reset_template = templates.get('password_reset',
            'thorson_wiki/password_reset.html')
    password_reset_confirm_template = templates.get('password_reset_confirm',
            'thorson_wiki/password_reset_confirm.html')

    landing_page = slugify(
            namespace.get('landing', namespaces.get('title', 'Main page'))
    )

    return [
        # Landing page
        url(r'^$', views.article_detail,
            {'namespace_': namespace_, 'slug': landing_page}, name='main'),

        # Authentication
        url(r'^user/login/$', auth_views.login,
            {'template_name': login_template, 'extra_context': {'namespace_':
                namespace_, 'namespace': namespace}}, name='login'),
        url(r'^user/logout/$', auth_views.logout, name='logout'),
        url(r'^user/password_change/$', auth_views.password_change,
            {'template_name': password_change_template, 'extra_context':
                {'namespace_': namespace_, 'namespace': namespace}},
            name='password_change'),
        url(r'^user/password_reset/$', auth_views.password_reset,
            {'template_name': password_reset_template, 'extra_context':
                {'namespace_': namespace_, 'namespace': namespace}}),
        url(r'^user/password_confirm/$', auth_views.password_reset_confirm,
            {'template_name': 'thorson_wiki/password_reset_confirm.html',
                'extra_context': {'namespace_': namespace_, 'namespace':
                    namespace}},
            name='password_reset_confirm'),

        # User info
        url(r'^user/register/$', views.register, {'namespace_': namespace_},
            name='user_create'),
        url(r'^user/update/$', views.user_update, {'namespace_': namespace_},
            name='user_update'),

        # Search
        url(r'^search/$', views.search, {'namespace_': namespace_},
            name='search'),

        # Articles
        url(r'^article/read/(?P<slug>[a-zA-Z0-9-_]+)/$',
             views.article_detail, {'namespace_': namespace_},
             name='article'),
        url(r'^article/update/(?P<slug>[a-zA-Z0-9-_]+)/$',
            views.article_update, {'namespace_': namespace_},
            name='article_update'),
        url(r'^article/delete/(?P<slug>[a-zA-Z0-9-_]+)/$',
            views.article_delete, {'namespace_': namespace_},
            name='article_delete'),
        url(r'^article/create/$', views.article_create,
            {'namespace_': namespace_}, name='article_create'),
        url(r'^article/list/$', views.article_list,
             {'namespace_': namespace_}, name='article_list'),
    ]



from haystack import indexes
from thorson_wiki.models import Article

class ArticleIndex(indexes.SearchIndex, indexes.Indexable):

    text = indexes.CharField(document=True, use_template=True)
    namespace = indexes.CharField(model_attr='namespace')

    def get_model(self):

        return Article



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-01-04 22:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='The title of this article.', max_length=255, verbose_name='title')),
                ('slug', models.SlugField(blank=True, editable=False, help_text="The article's title,  slugified for URLs.", max_length=255, verbose_name='slug')),
                ('namespace', models.CharField(choices=[('rivenrock', 'rivenrock'), ('srd5', 'srd5')], help_text="The article's namespace.", max_length=255, verbose_name='namespace')),
                ('content', models.TextField(blank=True, help_text='The content of the article.', verbose_name='content')),
                ('login_required', models.BooleanField(default=False, help_text='Whether only logged-in users may read the article.', verbose_name='login required')),
                ('redirect', models.CharField(blank=True, help_text='Allows this article to redirect to another one.', max_length=255, verbose_name='redirect to')),
            ],
            options={
                'verbose_name': 'article',
                'verbose_name_plural': 'articles',
            },
        ),
    ]



import markdown

class ThorsonMarkdown(markdown.Markdown):

    def __init__(self, *args, **kwargs):

        super(ThorsonMarkdown, self).__init__(*args, **kwargs)

        self.namespace = kwargs.get('namespace', None)
        self.namespace_ = kwargs.get('namespace_', None)
        self.request = kwargs.get('request', None)



from django import forms
from django.contrib import admin
from django.db import models
from django.utils.translation import ugettext as _
from thorson_wiki.models import *

font_family = "Menlo, Monaco, Consolars, \"Courier New\", monospace"

class ArticleAdmin(admin.ModelAdmin):
    """
    Represents the Article model on the admin site.
    """

    # The fields to be displayed
    list_display = ('title', 'namespace', 'public')
    list_filter = ('namespace', 'public')

    fieldsets = (
        (None, {'fields': ('title', 'content')}),
        (_("Meta"), {'fields': ('namespace', 'public', 'redirect')})
    )

    search_fields = ('title', 'content')
    ordering = ('title', 'namespace', 'public')
    filter_horizontal = ()

    formfield_overrides = {
        models.TextField: {
            'widget': forms.Textarea({
                'style': "font-family: %s;" % font_family,
                'class': 'vLargeTextField',
                'rows': '10',
                'cols': '40',
            }),
        },
    }

admin.site.register(Article, ArticleAdmin)



from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse_lazy
from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin, UserPassesTestMixin
from django.contrib.auth.models import Permission
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.views.generic.base import ContextMixin
from django.views.generic.detail import SingleObjectMixin, DetailView
from django.views.generic.edit import CreateView, DeleteView, UpdateView
from django.views.generic.list import ListView
from haystack.generic_views import SearchView
from thorson_wiki.forms import *
from thorson_wiki.models import Article

class NamespaceMixin(ContextMixin):
    """
    A shortcut mixin which adds the namespace to the context.
    """

    def dispatch(self, *args, **kwargs):

        self.namespace_ = kwargs.get('namespace_', '')

        if self.namespace_ not in settings.THORSONWIKI_NAMESPACES:
            raise Http404("Invalid namespace: %s" % self.namespace_)

        self.namespace = settings.THORSONWIKI_NAMESPACES[self.namespace_]

        return super(NamespaceMixin, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):

        context = super(NamespaceMixin, self).get_context_data(**kwargs)

        context['namespace'] = self.namespace
        context['namespace_'] = self.namespace_
        context['namespaces'] = settings.THORSONWIKI_NAMESPACES

        return context

class ArticleViewMixin(NamespaceMixin, SingleObjectMixin):
    """
    A shortcut mixin to use for all object views which operate on Articles.
    """

    model = Article

    def get_queryset(self):

        namespace_ = self.kwargs.get('namespace_', '')

        return self.model.objects.filter(namespace=namespace_)

class ArticleDetailView(DetailView, ArticleViewMixin):
    """
    Displays details on a single article.
    """

    def get_template_names(self):

        templates = settings.THORSONWIKI_NAMESPACES[self.namespace_].get(
            'templates', {}
        )

        temps = templates.get('detail', 'thorson_wiki/article_detail.html')

        if not isinstance(temps, list):
            temps = [temps]

        return temps

    def get(self, request, *args, **kwargs):

        try:
            return super(ArticleDetailView, self).get(request,
                    *args, **kwargs)
        except Http404:
            try:
                namespace = self.namespace_
            except KeyError:
                raise Http404

            return redirect(namespace + ':article_create')

    def render_to_response(self, context, **response_kwargs):

        if self.object.redirect:
            return redirect(self.namespace_ + ':article',
                    slug=slugify(self.object.redirect))
        else:
            return super(ArticleDetailView, self).render_to_response(
                context, **response_kwargs
            )

class ArticleUpdateView(UserPassesTestMixin, UpdateView, ArticleViewMixin):
    """
    Allows users to update articles.
    """

    form_class = ArticleUpdateForm

    def dispatch(self, request, *args, **kwargs):

        self.kwargs = kwargs
        self.object = self.get_object()

        return super(ArticleUpdateView, self).dispatch(request, *args,
                **kwargs)

    def get_template_names(self):
        
        templates = settings.THORSONWIKI_NAMESPACES[self.namespace_].get(
            'templates', {}
        )

        temps = templates.get('update', 'thorson_wiki/article_update.html')

        if not isinstance(temps, list):
            temps = [temps]

        return temps

    def test_func(self):

        return self.object.can_edit(self.request.user)

class ArticleCreateView(LoginRequiredMixin, NamespaceMixin, CreateView):
    """
    Allows users to create articles.
    """

    form_class = ArticleUpdateForm

    def get_template_names(self):

        templates = settings.THORSONWIKI_NAMESPACES[self.namespace_].get(
            'templates', {}
        )

        temps = templates.get('create', 'thorson_wiki/article_create.html')

        if not isinstance(temps, list):
            temps = [temps]

        return temps

    def get_initial(self):

        initial = super(ArticleCreateView, self).get_initial()

        initial['namespace'] = self.namespace_

        return initial

    def get_object(self):

        return Article(namespace=self.namespace_)

    def get_success_url(self):

        return self.object.get_absolute_url()

class ArticleDeleteView(UserPassesTestMixin, DeleteView, ArticleViewMixin):
    """
    Allows users to delete articles.
    """

    def dispatch(self, request, *args, **kwargs):

        # self.object needs to be available before test_func is called
        self.kwargs = kwargs
        self.object = self.get_object()

        return super(ArticleDeleteView, self).dispatch(request, *args,
                **kwargs)

    def test_func(self):

        return self.object.can_edit(self.request.user)

    def get_success_url(self):

        return reverse_lazy(self.namespace_ + ':main')

class ArticleListView(NamespaceMixin, ListView):
    """
    Allows users to view all articles.
    """

    model = Article

    def get_template_names(self):

        templates = settings.THORSONWIKI_NAMESPACES[self.namespace_].get(
            'templates', {}
        )

        temps = templates.get('list', 'thorson_wiki/article_list.html')

        if not isinstance(temps, list):
            temps = [temps]

        return temps

    def get_queryset(self):

        queryset = super(ArticleListView, self).get_queryset()
        queryset = queryset.filter(namespace=self.namespace_)
        queryset = queryset.order_by('title')

        return queryset

# USERS

class UserCreateView(CreateView, NamespaceMixin):

    model = User
    form_class = UserCreateForm

    def dispatch(self, request, *args, **kwargs):

        self.redirect_to = self.get_redirect_url(request, *args, **kwargs)

        return super(UserCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):

        context = super(UserCreateView, self).get_context_data(**kwargs)

        context['next'] = self.redirect_to

        return context

    def get_redirect_url(self, request, *args, **kwargs):

        namespace_ = kwargs.get('namespace_', '')

        return reverse_lazy(namespace_ + ':login')

    def get_success_url(self):

        return self.redirect_to

    def get_template_names(self):

        templates = settings.THORSONWIKI_NAMESPACES[self.namespace_].get(
            'templates', {}
        )

        temps = templates.get('user_create', 'thorson_wiki/user_create.html')

        if not isinstance(temps, list):
            temps = [temps]

        return temps

class UserUpdateView(UpdateView, NamespaceMixin):

    model = User
    form_class = UserUpdateForm

    def get_object(self, queryset=None):

        return self.request.user

    def get_success_url(self):

        return reverse_lazy(self.namespace_ + ':user_update')

    def get_template_names(self):

        templates = settings.THORSONWIKI_NAMESPACES[self.namespace_].get(
            'templates', {}
        )

        temps = templates.get('user_update', 'thorson_wiki/user_update.html')

        if not isinstance(temps, list):
            temps = [temps]

        return temps

class ArticleSearchView(SearchView, NamespaceMixin):

    def get_queryset(self):

        if self.request.method == 'GET':
            namespace_ = self.request.GET.get('namespace', self.namespace_)
        else:
            namespace_ = self.namespace_

        queryset = super(ArticleSearchView, self).get_queryset()

        if namespace_ == 'ALL':
            pass
        else:
            queryset = queryset.filter(namespace__iexact=namespace_)

        return queryset

    def get_template_names(self):

        templates = settings.THORSONWIKI_NAMESPACES[self.namespace_].get(
            'templates', {}
        )

        temps = templates.get('search', 'thorson_wiki/search.html')

        if not isinstance(temps, list):
            temps = [temps]

        return temps

article_detail = ArticleDetailView.as_view()
article_create = ArticleCreateView.as_view()
article_update = ArticleUpdateView.as_view()
article_delete = ArticleDeleteView.as_view()
article_list = ArticleListView.as_view()

register = UserCreateView.as_view()
user_update = UserUpdateView.as_view()

search = ArticleSearchView.as_view()
